﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using Microsoft.IdentityModel.Tokens;
using Microsoft.Extensions.Options;

namespace AccountRepository
{
    public class AuthenticationTools : IAuthenticationTools
    {
        public static readonly List<Person> AllUsers = new List<Person>();
        private readonly JwtSettings _jwtSettings;
        public AuthenticationTools(IOptions<JwtSettings> appSettings)
        {
            _jwtSettings = appSettings.Value;
        }

        public AuthenticationTools()
        {
        }

        public IEnumerable<Person> GetAll()
        {
            return AllUsers;
        }

        public Person GetById(int id)
        {
            return AllUsers.FirstOrDefault(x => x.ID == id);
        }

        public AuthenticateResponse Authenticate(AuthenticateRequest model)
        {
            var user = AllUsers.SingleOrDefault(x => x.ShownName == model.Username && x.Password == model.Password);

            // return null if user not found
            if (user == null) return null;

            // authentication successful so generate jwt token
            var token = GenerateJwtToken(user);

            return new AuthenticateResponse(user, token);
        }

        private string GenerateJwtToken(Person user)
        {
            var tokenHandler = new JwtSecurityTokenHandler();
            var bytes = Encoding.ASCII.GetBytes(_jwtSettings.Secret);
            var tokenDescriptor = new SecurityTokenDescriptor
            {
                Subject = new ClaimsIdentity(new[] { new Claim("id", user.ID.ToString()) }),
                Expires = DateTime.UtcNow.AddHours(1),
                SigningCredentials = new SigningCredentials(new SymmetricSecurityKey(bytes), SecurityAlgorithms.HmacSha256Signature)
            };
            var token = tokenHandler.CreateToken(tokenDescriptor);
            return tokenHandler.WriteToken(token);
        }
    }
}